from machine import Pin, I2C, SPI
import machine
import gc
import os
import sys

sys.path.insert(0, '/lib/hwModules')

import sdcard
import ds1307
import ssd1306
import tmp102


user_led=Pin(25,Pin.OUT)
user_led.value(0)

sw1=Pin(3,Pin.IN)
sw2=Pin(2,Pin.IN)

i2c = I2C(0, sda=Pin(4), scl=Pin(5),freq=100_000)

srq = Pin(20, Pin.IN, Pin.PULL_UP)						# Pi-Plate SRQ

GPIOpresent=[0,0,0,0,0,1,1,0,1,0,0,0,1,1,1,1,1,1,0,1,1,1,0,0,0,0,1]
GPIOmap=[0,0,0,0,0,28,6,0,27,0,0,0,15,7,0,1,24,29,0,8,26,14,0,0,0,0,9]
GPIOxref=27*[0]

sdPresent=True

#==============================================================================#
# HELP Functions                                                               #
#==============================================================================#
helpPath="lib/helpFiles/MICROPIhelp.txt"

def Help():
    help()

def HELP():
    help()
    
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        input('press \"Enter\" for more...')
                else:
                    Count=100
                    valid=False
        f.close()
    except:
        print ("No help file found.")

#LED Functions
def setLED():
    user_led.value(1)
    
def clrLED():
    user_led.value(0)
    
def toggleLED():
    if (user_led.value()):
        user_led.value(0)
    else:
        user_led.value(1)

#Switch Functions
SW1_int=False
SW2_int=False

def callback1(sw1):
    global SW1_int
    SW1_int=True

def callback2(sw2):
    global SW2_int    
    SW2_int=True

sw1.irq(trigger=Pin.IRQ_FALLING, handler=callback1)
sw2.irq(trigger=Pin.IRQ_FALLING, handler=callback2)


def getBUTTON(sw):
    global SW1_int, SW2_int
    val=False
    if (sw==1):
        #val=sw1.value()
        if(SW1_int):
            val=True
            SW1_int=False
    else:    
        #val=sw2.value()
        if(SW2_int):
            val=True        
            SW2_int=False
    return val


#Temperature Functions
def getTEMPC():
    return thermo.temperature

def getTEMPF():
    return thermo.temperature * 1.8 + 32.0

def getTEMPK():
    return thermo.temperature + 273.15


#Date and Time Functions
def getTIME():
    return rtc.gettime()

def getDATE():
    return rtc.getdate()

def setTIME():
    rtc.settime()
    
def getDATETIME():
    return rtc.datetime()

def getDAY():
    return rtc.getday()

#GPIO Functions
def configGPIO(chan, direction, pull=None):
    checkCHAN(chan)
    direction=direction.lower()
    assert (direction == "in" or direction == 'out'), "Error: direction must either 'in' or 'out'."
    if (direction=='out'):
        GPIOxref[chan] = Pin(GPIOmap[chan], Pin.OUT)
    else:
        assert (pullup==None or pullup==True or pullup==False), "Invalid 'pullup' argument - must be True or False"
        if (pullup==False or pullup==None):
            GPIOxref[chan] = Pin(GPIOmap[chan], Pin.IN)
        else:
            GPIOxref[chan] = Pin(GPIOmap[chan], Pin.IN, Pin.PULL_UP)                  

def initGPIO():
    for i in range(27):
        if (GPIOpresent[i]):
            GPIOxref[i] = Pin(GPIOmap[i], Pin.IN)

def getGPIO(chan):
    checkCHAN(chan)
    return GPIOxref[chan].value()

def setGPIO(chan,val=None):
    checkCHAN(chan)
    assert (val==None or val==0 or val==1 or val==False or val==True), "Ivalid 'value' argument. Must be 0, 1, True, or False."
    if (val==None or val==True or val==1):
        GPIOxref[chan].value(1)
    else:
        GPIOxref[chan].value(0)

def clrGPIO(chan):
    checkCHAN(chan)
    GPIOxref[chan].value(0)
    pass

def toggleGPIO(chan):
    checkCHAN(chan)
    if (GPIOxref[chan].value()):
       GPIOxref[chan].value(0)
    else:
        GPIOxref[chan].value(1)

def getSRQ():
    resp = False
    if (srq.value()==0):
        resp = True
    return resp

def checkCHAN(chan):
    assert (chan>0 and chan<27), "GPIO port value out of range. must be between 0 and 26" 
    assert (GPIOpresent[chan] == 1), "Invalid GPIO port." 

#memory functions
def getFREEMEM():
    mem=2*[0]
    flash=os.statvfs('/')
    mem[0]=int(flash[0]*flash[3]/1024)
    gc.collect()
    mem[1]=gc.mem_free()
    return mem
    
#Display Functions
dispLines=3*[0]

def dispWriteline(text,line):
    assert (line>0 and line<4), 'Line number must be between 1 and 3'
    assert (isinstance(text,str)), 'Only strings can be sent to this function'
    dispLines[line-1]=text
    display.fill(0)
    for i in range(3):
        display.text(dispLines[i],0,i*10)
    display.show()
    
def dispWriteall(textList):
    assert (len(textList) == 3), 'The function require 3 lines of text'
    for i in range(3):
        assert (isinstance(textList[i],str)), 'Only strings can be sent to this function'
    display.fill(0)
    for i in range(3):
        dispLines[i]=textList[i]
        display.text(dispLines[i],0,i*10)
    display.show()
        
initGPIO()	#Configure all undedicated GPIO lines as inputs

#Load up peripheral modules
try:
    sd = sdcard.SDCard()					#Load SD card library
except:
    sdPresent=False
    sd = 0
display = ssd1306.SSD1306_I2C(128, 32, i2c)	#Load OLED display module
rtc = ds1307.DS1307(i2c)					#Load real time clock module
thermo=tmp102.Tmp102(i2c, 0x48)	

#Display welcome message
mem=getFREEMEM()
msgText = 3*[0]
msgText[0] = 'MICROpi'
msgText[1] = 'Free RAM:'+str(mem[1])
msgText[2] = 'Free Mem:'+str(mem[0])+'KB'
dispWriteall(msgText)

#Do garbage collection
gc.collect()