from machine import SPI, Pin
import time

spi_cs = Pin(17, Pin.OUT, value=1)        	# SPI CS = GP20
frame = Pin(21, Pin.OUT, value=0)        	# Pi-Plate Frame = GP6
ack = Pin(22, Pin.IN)						# Pi-Plate ACK=GP27
srq = Pin(20, Pin.IN)						# Pi-Plate SRQ=GP9
spi = SPI(0,                               	# SPI GP6=SCK, GP7=MOSI, GP8=MISO
    baudrate=500000,
    polarity=0,
    phase=0,
    bits=8,
    firstbit=SPI.MSB,
    sck=Pin(18),
    mosi=Pin(19),
    miso=Pin(16))
sw_in = Pin(23, Pin.IN)

DataGood=True

def ppCMD1(addr,cmd,param1,param2,bytes2return):
    arg = bytearray()
    arg.append(addr)   
    arg.append(cmd) 
    arg.append(param1)  
    arg.append(param2)
    frame.value( 1 )
    spi_cs.value(0)
    spi.write( arg )
    time.sleep_us(80)
    spi_cs.value(1)
    resp = [0]*(bytes2return+1)
    if bytes2return>0:
        time.sleep_us(100)
        for i in range(0,bytes2return):	#Fetch each byte.
            spi_cs.value(0)
            dummy = spi.read(1)
            time.sleep_us(10)
            spi_cs.value(1)              
            resp[i]=int.from_bytes(dummy, "big")         
    time.sleep_us(100)
    frame.value( 0 )
    #time.sleep_us(10)
    return resp	

def ppCMDer(addr,cmd,param1,param2,bytes2return):
    #global RELAY2baseADDR
    global DataGood
    DataGood=True
    arg = bytearray()
    arg.append(addr)   
    arg.append(cmd) 
    arg.append(param1)  
    arg.append(param2) 
    frame.value( 1 )
    spi_cs.value(0)
    spi.write( arg )
    spi_cs.value(1)
    resp = [0]*(bytes2return+1)
    DataGood=True
    t0=time.ticks_ms()
    wait=True
    while(wait):
        if (ack.value()!=1):  #wait up to 50msec for the addressed DAQC2 to ACKnowledge command
            wait=False
        if (time.ticks_diff(time.ticks_ms(), t0)>50):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If DAQC2 is supposed to send data AND no timeout occurred
        wait=True
        t0=time.ticks_ms()
        while(wait):
            if (ack.value()!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if (time.ticks_diff(time.ticks_ms(), t0)>80): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data
            #time.sleep(.0001)
            spi_cs.value(0) 
            for i in range(0,bytes2return+1):	#Fetch each byte. 
                dummy = spi.read(1)
                resp[i]=int.from_bytes(dummy, "big")
            spi_cs.value(1)        
            csum=0;                             
            for i in range(0,bytes2return):     #calculate and verify checksum
                csum+=resp[i]
            if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                DataGood=False
    frame.value( 0 )
    return resp

def ppCMDosc(addr,cmd,param1,param2,bytes2return):
    #global RELAY2baseADDR
    global DataGood
    DataGood=True
    arg = bytearray()
    arg.append(addr)   
    arg.append(cmd) 
    arg.append(param1)  
    arg.append(param2) 
    frame.value( 1 )
    spi_cs.value(0)
    spi.write( arg )
    spi_cs.value(1)
    #dummy = [0]*(bytes2return)
    resp = [0]*(bytes2return)
    DataGood=True
    t0=time.ticks_ms()
    wait=True
    while(wait):
        if (ack.value()!=1):  #wait up to 50msec for the addressed DAQC2 to ACKnowledge command
            wait=False
        if (time.ticks_diff(time.ticks_ms(), t0)>50):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If DAQC2 is supposed to send data AND no timeout occurred
        wait=True
        t0=time.ticks_ms()
        while(wait):
            if (ack.value()!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if (time.ticks_diff(time.ticks_ms(), t0)>80): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data
            spi_cs.value(0) 
            for i in range(0,bytes2return):	#Fetch each byte. 
                dummy = spi.read(1)
                resp[i]=int.from_bytes(dummy, "big")
            spi_cs.value(1) 
    frame.value( 0 )
    return resp

def fetchBLOCK(addr,cmd,param1,param2,bytes2return):
    global DataGood
    #resp=bytes2return*[0]
    arg = bytearray()
    arg.append(addr)   
    arg.append(cmd) 
    arg.append(param1)  
    arg.append(param2) 
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (ack.value()!=0):  #Ensure that ACK is high before proceeding
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    frame.value( 1 )
    time.sleep_us(10)
    spi_cs.value(0)
    spi.write( arg )
    spi_cs.value(1)
    t0=time.time()
    wait=True
    while(wait):
        if (ack.value()==0):  #wait until the command is received and ACKnowledged
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if DataGood:
        spi_cs.value(0)
        time.sleep_us(10)     #allow the uP some time to initialize the SPI
        blockBYTES=spi.read(bytes2return)
        spi_cs.value(1)
    frame.value( 0 )
    return list(blockBYTES)